import 'dart:core';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:quickblox_sdk/chat/constants.dart';
import 'package:quickblox_sdk/models/qb_dialog.dart';
import 'package:quickblox_sdk/models/qb_filter.dart';
import 'package:quickblox_sdk/models/qb_message.dart';
import 'package:quickblox_sdk/models/qb_sort.dart';
import 'package:quickblox_sdk/quickblox_sdk.dart';
import 'package:quickblox_sdk_example/credentials.dart';
import 'package:quickblox_sdk_example/utils/dialog_utils.dart';
import 'package:quickblox_sdk_example/utils/snackbar_utils.dart';

class ChatScreen extends StatefulWidget {
  @override
  State<StatefulWidget> createState() => _ChatScreenState();
}

class _ChatScreenState extends State<ChatScreen> {
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();

  String dialogId = DIALOG_ID;
  String messageId;

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        key: _scaffoldKey,
        appBar: AppBar(
          title: const Text('Chat'),
          centerTitle: true,
          leading: new IconButton(
              icon: new Icon(Icons.arrow_back),
              onPressed: () => Navigator.of(context).pop()),
        ),
        body: Center(
            child: SingleChildScrollView(
                child: Column(children: [
          MaterialButton(
              minWidth: 200,
              onPressed: connect,
              child: Text('connect'),
              color: Theme.of(context).primaryColor,
              textColor: Colors.white),
          MaterialButton(
              minWidth: 200,
              onPressed: disconnect,
              child: Text('disconnect'),
              color: Theme.of(context).primaryColor,
              textColor: Colors.white),
          MaterialButton(
              minWidth: 200,
              onPressed: isConnected,
              child: Text('is connected'),
              color: Theme.of(context).primaryColor,
              textColor: Colors.white),
          MaterialButton(
              minWidth: 200,
              onPressed: pingServer,
              child: Text('ping server'),
              color: Theme.of(context).primaryColor,
              textColor: Colors.white),
          MaterialButton(
              minWidth: 200,
              onPressed: pingUser,
              child: Text('ping user'),
              color: Theme.of(context).primaryColor,
              textColor: Colors.white),
          MaterialButton(
              minWidth: 200,
              onPressed: getDialogs,
              child: Text('get dialogs'),
              color: Theme.of(context).primaryColor,
              textColor: Colors.white),
          MaterialButton(
              minWidth: 200,
              onPressed: getDialogsCount,
              child: Text('get dialogs count'),
              color: Theme.of(context).primaryColor,
              textColor: Colors.white),
          MaterialButton(
              minWidth: 200,
              onPressed: updateDialog,
              child: Text('update dialog'),
              color: Theme.of(context).primaryColor,
              textColor: Colors.white),
          MaterialButton(
              minWidth: 200,
              onPressed: createDialog,
              child: Text('create dialog'),
              color: Theme.of(context).primaryColor,
              textColor: Colors.white),
          MaterialButton(
              minWidth: 200,
              onPressed: deleteDialog,
              child: Text('delete dialog'),
              color: Theme.of(context).primaryColor,
              textColor: Colors.white),
          MaterialButton(
              minWidth: 200,
              onPressed: leaveDialog,
              child: Text('leave dialog'),
              color: Theme.of(context).primaryColor,
              textColor: Colors.white),
          MaterialButton(
              minWidth: 200,
              onPressed: joinDialog,
              child: Text('join dialog'),
              color: Theme.of(context).primaryColor,
              textColor: Colors.white),
          MaterialButton(
              minWidth: 200,
              onPressed: getOnlineUsers,
              child: Text('get online users'),
              color: Theme.of(context).primaryColor,
              textColor: Colors.white),
          MaterialButton(
              minWidth: 200,
              onPressed: sendMessage,
              child: Text('send message'),
              color: Theme.of(context).primaryColor,
              textColor: Colors.white),
          MaterialButton(
              minWidth: 200,
              onPressed: sendSystemMessage,
              child: Text('send system message'),
              color: Theme.of(context).primaryColor,
              textColor: Colors.white),
          MaterialButton(
              minWidth: 200,
              onPressed: () {
                subscribeMessageEvents(QBChatEvents.RECEIVED_NEW_MESSAGE);
                subscribeMessageEvents(
                    QBChatEvents.RECEIVED_NEW_SYSTEM_MESSAGE);
              },
              child: Text('subscribe message events'),
              color: Theme.of(context).primaryColor,
              textColor: Colors.white),
          MaterialButton(
              minWidth: 200,
              onPressed: () {
                unsubscribeMessageEvents(QBChatEvents.RECEIVED_NEW_MESSAGE);
                unsubscribeMessageEvents(
                    QBChatEvents.RECEIVED_NEW_SYSTEM_MESSAGE);
              },
              child: Text('unsubscribe message events'),
              color: Theme.of(context).primaryColor,
              textColor: Colors.white),
          MaterialButton(
              minWidth: 200,
              onPressed: markMessageRead,
              child: Text('mark message read'),
              color: Theme.of(context).primaryColor,
              textColor: Colors.white),
          MaterialButton(
              minWidth: 200,
              onPressed: markMessageDelivered,
              child: Text('mark message delivered'),
              color: Theme.of(context).primaryColor,
              textColor: Colors.white),
          MaterialButton(
              minWidth: 200,
              onPressed: () {
                subscribeMessageStatus(QBChatEvents.MESSAGE_DELIVERED);
                subscribeMessageStatus(QBChatEvents.MESSAGE_READ);
              },
              child: Text('subscribe message status'),
              color: Theme.of(context).primaryColor,
              textColor: Colors.white),
          MaterialButton(
              minWidth: 200,
              onPressed: () {
                unsubscribeMessageStatus(QBChatEvents.MESSAGE_DELIVERED);
                unsubscribeMessageStatus(QBChatEvents.MESSAGE_READ);
              },
              child: Text('unsubscribe message status'),
              color: Theme.of(context).primaryColor,
              textColor: Colors.white),
          MaterialButton(
              minWidth: 200,
              onPressed: sendIsTyping,
              child: Text('send is typing'),
              color: Theme.of(context).primaryColor,
              textColor: Colors.white),
          MaterialButton(
              minWidth: 200,
              onPressed: sendStoppedTyping,
              child: Text('send stopped typing'),
              color: Theme.of(context).primaryColor,
              textColor: Colors.white),
          MaterialButton(
              minWidth: 200,
              onPressed: () {
                subscribeIsTyping(QBChatEvents.PROCESS_USER_IS_TYPING);
                subscribeStopTyping(QBChatEvents.PROCESS_USER_STOP_TYPING);
              },
              child: Text('subscribe typing'),
              color: Theme.of(context).primaryColor,
              textColor: Colors.white),
          MaterialButton(
              minWidth: 200,
              onPressed: () {
                unsubscribeTyping(QBChatEvents.PROCESS_USER_IS_TYPING);
                unsubscribeTyping(QBChatEvents.PROCESS_USER_STOP_TYPING);
              },
              child: Text('unsubscribe typing'),
              color: Theme.of(context).primaryColor,
              textColor: Colors.white),
          MaterialButton(
              minWidth: 200,
              onPressed: getDialogMessages,
              child: Text('get dialog messages'),
              color: Theme.of(context).primaryColor,
              textColor: Colors.white),
          MaterialButton(
              minWidth: 200,
              onPressed: () {
                subscribeEventConnections(QBChatEvents.CONNECTED);
                subscribeEventConnections(QBChatEvents.CONNECTION_CLOSED);
                subscribeEventConnections(QBChatEvents.RECONNECTION_FAILED);
                subscribeEventConnections(QBChatEvents.RECONNECTION_SUCCESSFUL);
              },
              child: Text('subscribe event connections'),
              color: Theme.of(context).primaryColor,
              textColor: Colors.white),
          MaterialButton(
              minWidth: 200,
              onPressed: () {
                unsubscribeEventConnections(QBChatEvents.CONNECTED);
                unsubscribeEventConnections(QBChatEvents.CONNECTION_CLOSED);
                unsubscribeEventConnections(QBChatEvents.RECONNECTION_FAILED);
                unsubscribeEventConnections(
                    QBChatEvents.RECONNECTION_SUCCESSFUL);
              },
              child: Text('unsubscribe event connections'),
              color: Theme.of(context).primaryColor,
              textColor: Colors.white),
        ]))));
  }

  void connect() async {
    try {
      await QB.chat.connect(USER_ID, USER_PASSWORD);
      SnackBarUtils.showResult(_scaffoldKey, "The chat was connected");
    } on PlatformException catch (e) {
      DialogUtils.showError(context, e);
    }
  }

  void disconnect() async {
    try {
      await QB.chat.disconnect();
      SnackBarUtils.showResult(_scaffoldKey, "The chat was disconnected");
    } on PlatformException catch (e) {
      DialogUtils.showError(context, e);
    }
  }

  void isConnected() async {
    try {
      bool connected = await QB.chat.isConnected();
      SnackBarUtils.showResult(
          _scaffoldKey, "The Chat connected: " + connected.toString());
    } on PlatformException catch (e) {
      DialogUtils.showError(context, e);
    }
  }

  void pingServer() async {
    try {
      await QB.chat.pingServer();
      SnackBarUtils.showResult(_scaffoldKey, "The server was ping success");
    } on PlatformException catch (e) {
      DialogUtils.showError(context, e);
    }
  }

  void pingUser() async {
    try {
      await QB.chat.pingUser(USER_ID);
      SnackBarUtils.showResult(
          _scaffoldKey, "The user $USER_ID was ping success");
    } on PlatformException catch (e) {
      DialogUtils.showError(context, e);
    }
  }

  void getDialogs() async {
    try {
      QBSort sort = QBSort();
      sort.field = QBChatDialogSorts.LAST_MESSAGE_DATE_SENT;
      sort.ascending = true;

      QBFilter filter = QBFilter();
      filter.field = QBChatDialogFilterFields.LAST_MESSAGE_DATE_SENT;
      filter.operator = QBChatDialogFilterOperators.ALL;
      filter.value = "";

      int limit = 100;
      int skip = 0;

      List<QBDialog> dialogs = await QB.chat
          .getDialogs(sort: sort, filter: filter, limit: limit, skip: skip);
      var dialogsLength = dialogs.length;
      SnackBarUtils.showResult(
          _scaffoldKey, "The $dialogsLength dialogs were loaded success");
    } on PlatformException catch (e) {
      DialogUtils.showError(context, e);
    }
  }

  void getDialogsCount() async {
    try {
      QBFilter filter = QBFilter();
      filter.field = QBChatDialogFilterFields.LAST_MESSAGE_DATE_SENT;
      filter.operator = QBChatDialogFilterOperators.ALL;
      filter.value = "";

      int limit = 100;
      int skip = 0;

      var dialogsCount = await QB.chat
          .getDialogsCount(qbFilter: filter, limit: limit, skip: skip);
      SnackBarUtils.showResult(
          _scaffoldKey, "The dialogs count is: $dialogsCount");
    } on PlatformException catch (e) {
      DialogUtils.showError(context, e);
    }
  }

  void updateDialog() async {
    try {
      String dialogId = DIALOG_ID;
      String dialogName = "FLUTTER_CHAT_updated_dialog";
      List<int> addUsers = new List();
      List<int> removeUsers = new List();

      QBDialog updatedDialog = await QB.chat.updateDialog(dialogId,
          addUsers: addUsers, removeUsers: removeUsers, dialogName: dialogName);

      String updatedDialogId = updatedDialog.id;
      SnackBarUtils.showResult(
          _scaffoldKey, "The dialog $updatedDialogId was updated");
    } on PlatformException catch (e) {
      DialogUtils.showError(context, e);
    }
  }

  void createDialog() async {
    List<int> occupantsIds = List.from(OPPONENTS_IDS);
    String dialogName =
        "FLUTTER_CHAT_" + new DateTime.now().millisecond.toString();

    int dialogType = QBChatDialogTypes.PUBLIC_CHAT;

    try {
      QBDialog createdDialog = await QB.chat
          .createDialog(occupantsIds, dialogName, dialogType: dialogType);
      dialogId = createdDialog.id;
      SnackBarUtils.showResult(
          _scaffoldKey, "The dialog $dialogId was created");
    } on PlatformException catch (e) {
      DialogUtils.showError(context, e);
    }
  }

  void deleteDialog() async {
    try {
      await QB.chat.deleteDialog(dialogId);
      SnackBarUtils.showResult(
          _scaffoldKey, "The dialog $dialogId was deleted");
      dialogId = null;
    } on PlatformException catch (e) {
      DialogUtils.showError(context, e);
    }
  }

  void leaveDialog() async {
    try {
      await QB.chat.leaveDialog(dialogId);
      SnackBarUtils.showResult(_scaffoldKey, "The dialog $dialogId was leaved");
    } on PlatformException catch (e) {
      DialogUtils.showError(context, e);
    }
  }

  void joinDialog() async {
    try {
      await QB.chat.joinDialog(dialogId);
      SnackBarUtils.showResult(_scaffoldKey, "The dialog $dialogId was joined");
    } on PlatformException catch (e) {
      DialogUtils.showError(context, e);
    }
  }

  void getOnlineUsers() async {
    try {
      List<int> onlineUsers = await QB.chat.getOnlineUsers(dialogId);
      SnackBarUtils.showResult(
          _scaffoldKey, "The online users are: $onlineUsers");
    } on PlatformException catch (e) {
      DialogUtils.showError(context, e);
    }
  }

  void sendMessage() async {
    String messageBody =
        "Hello from flutter!" + "\n From user: " + USER_ID.toString();
    try {
      await QB.chat
          .sendMessage(dialogId, body: messageBody, saveToHistory: true);
      SnackBarUtils.showResult(
          _scaffoldKey, "The message was sent to dialog: $dialogId");
    } on PlatformException catch (e) {
      DialogUtils.showError(context, e);
    }
  }

  void sendSystemMessage() async {
    try {
      await QB.chat.sendSystemMessage(OPPONENT_ID);
      SnackBarUtils.showResult(_scaffoldKey, "The system message was sent");
    } on PlatformException catch (e) {
      DialogUtils.showError(context, e);
    }
  }

  void subscribeMessageEvents(String eventName) async {
    try {
      await QB.chat.subscribeMessageEvents(QBChatEvents.RECEIVED_NEW_MESSAGE,
          (data) {
        Map<String, Object> map = new Map<String, dynamic>.from(data);
        String messageType = map["type"];
        String messageBody = "system message";
        if (messageType == QBChatEvents.RECEIVED_NEW_MESSAGE) {
          Map<String, Object> payload =
              new Map<String, dynamic>.from(map["payload"]);
          messageBody = payload["body"];
          messageId = payload["id"];
        }
        SnackBarUtils.showResult(
            _scaffoldKey, "Received message event: \n" + messageBody);
      });
      SnackBarUtils.showResult(
          _scaffoldKey,
          "Subscribed message event: " +
              eventName +
              "\nFor dialog: " +
              dialogId);
    } on PlatformException catch (e) {
      DialogUtils.showError(context, e);
    }
  }

  void unsubscribeMessageEvents(String eventName) async {
    try {
      await QB.chat.unsubscribeMessageEvents(eventName);
      SnackBarUtils.showResult(
          _scaffoldKey,
          "Unsubscribed message event: " +
              eventName +
              "\nFor dialog: " +
              dialogId);
    } on PlatformException catch (e) {
      DialogUtils.showError(context, e);
    }
  }

  void markMessageRead() async {
    QBMessage qbMessage = new QBMessage();
    qbMessage.dialogId = dialogId;
    qbMessage.id = messageId;
    qbMessage.senderId = USER_ID;

    try {
      await QB.chat.markMessageRead(qbMessage);
      SnackBarUtils.showResult(
          _scaffoldKey, "The message " + messageId + " was marked read");
    } on PlatformException catch (e) {
      DialogUtils.showError(context, e);
    }
  }

  void markMessageDelivered() async {
    QBMessage qbMessage = new QBMessage();
    qbMessage.dialogId = dialogId;
    qbMessage.id = messageId;
    qbMessage.senderId = USER_ID;

    try {
      await QB.chat.markMessageDelivered(qbMessage);
      SnackBarUtils.showResult(
          _scaffoldKey, "The message " + messageId + " was marked delivered");
    } on PlatformException catch (e) {
      DialogUtils.showError(context, e);
    }
  }

  void subscribeMessageStatus(String eventName) async {
    try {
      await QB.chat.subscribeMessageStatus(eventName, (data) {
        SnackBarUtils.showResult(_scaffoldKey, "Message status: " + data);
      });
      SnackBarUtils.showResult(
          _scaffoldKey,
          "Subscribed message status: " +
              eventName +
              "\nFor dialog: " +
              dialogId);
    } on PlatformException catch (e) {
      DialogUtils.showError(context, e);
    }
  }

  void unsubscribeMessageStatus(String eventName) async {
    try {
      await QB.chat.unsubscribeMessageStatus(eventName);
      SnackBarUtils.showResult(
          _scaffoldKey,
          "Unsubscribed message status: " +
              eventName +
              "\nFor dialog: " +
              dialogId);
    } on PlatformException catch (e) {
      DialogUtils.showError(context, e);
    }
  }

  void sendIsTyping() async {
    try {
      await QB.chat.sendIsTyping(dialogId);
      SnackBarUtils.showResult(
          _scaffoldKey, "Sent is typing for dialog: " + dialogId);
    } on PlatformException catch (e) {
      DialogUtils.showError(context, e);
    }
  }

  void sendStoppedTyping() async {
    try {
      await QB.chat.sendStoppedTyping(dialogId);
      SnackBarUtils.showResult(
          _scaffoldKey, "Sent stopped typing for dialog: " + dialogId);
    } on PlatformException catch (e) {
      DialogUtils.showError(context, e);
    }
  }

  void subscribeIsTyping(String eventName) async {
    try {
      await QB.chat.subscribeTyping(eventName, (data) {
        Map<String, Object> map = new Map<String, dynamic>.from(data);
        Map<String, Object> payload =
            new Map<String, dynamic>.from(map["payload"]);
        int userId = payload["userId"];
        SnackBarUtils.showResult(
            _scaffoldKey, "Typing user: " + userId.toString());
      });
      SnackBarUtils.showResult(_scaffoldKey,
          "Subscribed typing: " + eventName + "\nFor dialog: " + dialogId);
    } on PlatformException catch (e) {
      DialogUtils.showError(context, e);
    }
  }

  void subscribeStopTyping(String eventName) async {
    try {
      await QB.chat.subscribeTyping(eventName, (data) {
        Map<String, Object> map = new Map<String, dynamic>.from(data);
        Map<String, Object> payload =
            new Map<String, dynamic>.from(map["payload"]);
        int userId = payload["userId"];
        SnackBarUtils.showResult(
            _scaffoldKey, "Stopped typing user: " + userId.toString());
      });
      SnackBarUtils.showResult(_scaffoldKey,
          "Subscribed typing: " + eventName + "\nFor dialog: " + dialogId);
    } on PlatformException catch (e) {
      DialogUtils.showError(context, e);
    }
  }

  void unsubscribeTyping(String eventName) async {
    try {
      await QB.chat.unsubscribeTyping(eventName);
      SnackBarUtils.showResult(_scaffoldKey,
          "Unsubscribed typing: " + eventName + "\nFor dialog: " + dialogId);
    } on PlatformException catch (e) {
      DialogUtils.showError(context, e);
    }
  }

  void getDialogMessages() async {
    try {
      List<QBMessage> messages = await QB.chat.getDialogMessages(dialogId);
      int countMessages = messages.asMap().length;
      SnackBarUtils.showResult(
          _scaffoldKey, "Loaded messages: " + countMessages.toString());
    } on PlatformException catch (e) {
      DialogUtils.showError(context, e);
    }
  }

  void subscribeEventConnections(String eventName) async {
    try {
      await QB.chat.subscribeEventConnections(eventName, (data) {
        String type = data["type"];
        SnackBarUtils.showResult(
            _scaffoldKey, "Received event connection: $type");
      });
      SnackBarUtils.showResult(
          _scaffoldKey, "Subscribed to event connections: " + eventName);
    } on PlatformException catch (e) {
      DialogUtils.showError(context, e);
    }
  }

  void unsubscribeEventConnections(String eventName) async {
    try {
      await QB.chat.unsubscribeEventConnections(eventName);
      SnackBarUtils.showResult(
          _scaffoldKey, "Unsubscribed to event connections: " + eventName);
    } on PlatformException catch (e) {
      DialogUtils.showError(context, e);
    }
  }
}
