import 'dart:async';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:quickblox_sdk/file/constants.dart';
import 'package:quickblox_sdk/models/qb_file.dart';
import 'package:quickblox_sdk/quickblox_sdk.dart';
import 'package:quickblox_sdk_example/utils/dialog_utils.dart';
import 'package:quickblox_sdk_example/utils/snackbar_utils.dart';

class ContentScreen extends StatefulWidget {
  @override
  State<StatefulWidget> createState() => _ContentScreenState();
}

class _ContentScreenState extends State<ContentScreen> {
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        key: _scaffoldKey,
        appBar: AppBar(
          title: const Text('File'),
          centerTitle: true,
          leading: new IconButton(
              icon: new Icon(Icons.arrow_back),
              onPressed: () => Navigator.of(context).pop()),
        ),
        body: Center(
            child: Column(children: [
          MaterialButton(
            minWidth: 200,
            child: Text('subscribe upload progress'),
            color: Theme.of(context).primaryColor,
            textColor: Colors.white,
            onPressed: () {
              subscribeUpload(QBFileUploadProgress.FILE_UPLOAD_PROGRESS);
            },
          ),
          MaterialButton(
            minWidth: 200,
            child: Text('unsubscribe upload progress'),
            color: Theme.of(context).primaryColor,
            textColor: Colors.white,
            onPressed: () {
              unsubscribeUpload(QBFileUploadProgress.FILE_UPLOAD_PROGRESS);
            },
          ),
          MaterialButton(
            minWidth: 200,
            child: Text('upload'),
            color: Theme.of(context).primaryColor,
            textColor: Colors.white,
            onPressed: upload,
          ),
          MaterialButton(
            minWidth: 200,
            child: Text('get info'),
            color: Theme.of(context).primaryColor,
            textColor: Colors.white,
            onPressed: getInfo,
          ),
          MaterialButton(
            minWidth: 200,
            child: Text('get public URL'),
            color: Theme.of(context).primaryColor,
            textColor: Colors.white,
            onPressed: getPublicURL,
          ),
          MaterialButton(
            minWidth: 200,
            child: Text('get private URL'),
            color: Theme.of(context).primaryColor,
            textColor: Colors.white,
            onPressed: getPrivateURL,
          ),
        ])));
  }

  Future<void> subscribeUpload(String eventName) async {
    String url = "";

    try {
      await QB.content.subscribeUploadProgress(url, eventName, (data) {
        int progress = data["progress"];
        String url = data["url"];

        SnackBarUtils.showResult(
            _scaffoldKey, "Progress value is: $progress \n for url $url");
      });
      SnackBarUtils.showResult(_scaffoldKey, "");
    } on PlatformException catch (e) {
      DialogUtils.showError(context, e);
    }
  }

  Future<void> unsubscribeUpload(String eventName) async {
    String url = "";

    try {
      await QB.content.unsubscribeUploadProgress(url, eventName);
      SnackBarUtils.showResult(_scaffoldKey, "");
    } on PlatformException catch (e) {
      DialogUtils.showError(context, e);
    }
  }

  Future<void> upload() async {
    String url = "";

    try {
      QBFile file = await QB.content.upload(url);
      int id = file.id;
      SnackBarUtils.showResult(_scaffoldKey, "The file $id was uploaded");
    } on PlatformException catch (e) {
      DialogUtils.showError(context, e);
    }
  }

  Future<void> getInfo() async {
    int fileId = 0;

    try {
      QBFile file = await QB.content.getInfo(fileId);
      int id = file.id;
      SnackBarUtils.showResult(_scaffoldKey, "The file $id was loaded");
    } on PlatformException catch (e) {
      DialogUtils.showError(context, e);
    }
  }

  Future<void> getPublicURL() async {
    String uid = "";

    try {
      String url = await QB.content.getPublicURL(uid);
      SnackBarUtils.showResult(_scaffoldKey, "Url $url was loaded");
    } on PlatformException catch (e) {
      DialogUtils.showError(context, e);
    }
  }

  Future<void> getPrivateURL() async {
    String uid = "";

    try {
      String url = await QB.content.getPrivateURL(uid);
      SnackBarUtils.showResult(_scaffoldKey, "Url $url was loaded");
    } on PlatformException catch (e) {
      DialogUtils.showError(context, e);
    }
  }
}
