import 'dart:async';
import 'dart:core';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:quickblox_sdk/models/qb_rtc_session.dart';
import 'package:quickblox_sdk/quickblox_sdk.dart';
import 'package:quickblox_sdk/webrtc/constants.dart';
import 'package:quickblox_sdk/webrtc/rtc_video_view.dart';
import 'package:quickblox_sdk_example/credentials.dart';
import 'package:quickblox_sdk_example/utils/dialog_utils.dart';
import 'package:quickblox_sdk_example/utils/snackbar_utils.dart';

class WebRTCScreen extends StatefulWidget {
  @override
  State<StatefulWidget> createState() => _WebRTCScreenState();
}

class _WebRTCScreenState extends State<WebRTCScreen> {
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();

  String _sessionId;

  RTCVideoViewController _localVideoViewController;
  RTCVideoViewController _remoteVideoViewController;

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        key: _scaffoldKey,
        appBar: AppBar(
          title: const Text('WebRTC'),
          centerTitle: true,
          leading: new IconButton(
              icon: new Icon(Icons.arrow_back),
              onPressed: () => Navigator.of(context).pop()),
        ),
        body: Center(
            child: SingleChildScrollView(
                child: Column(children: [
          MaterialButton(
            minWidth: 200,
            child: Text('init WebRTC'),
            color: Theme.of(context).primaryColor,
            textColor: Colors.white,
            onPressed: init,
          ),
          MaterialButton(
            minWidth: 200,
            child: Text('release WebRTC'),
            color: Theme.of(context).primaryColor,
            textColor: Colors.white,
            onPressed: releaseWebRTC,
          ),
          MaterialButton(
            minWidth: 200,
            child: Text('get session WebRTC'),
            color: Theme.of(context).primaryColor,
            textColor: Colors.white,
            onPressed: getSessionWebRTC,
          ),
          MaterialButton(
            minWidth: 200,
            child: Text('call WebRTC'),
            color: Theme.of(context).primaryColor,
            textColor: Colors.white,
            onPressed: callWebRTC,
          ),
          MaterialButton(
            minWidth: 200,
            child: Text('accept WebRTC'),
            color: Theme.of(context).primaryColor,
            textColor: Colors.white,
            onPressed: acceptWebRTC,
          ),
          MaterialButton(
            minWidth: 200,
            child: Text('reject WebRTC'),
            color: Theme.of(context).primaryColor,
            textColor: Colors.white,
            onPressed: rejectWebRTC,
          ),
          MaterialButton(
            minWidth: 200,
            child: Text('hangUp WebRTC'),
            color: Theme.of(context).primaryColor,
            textColor: Colors.white,
            onPressed: hangUpWebRTC,
          ),
          MaterialButton(
            minWidth: 200,
            child: Text('enable video WebRTC'),
            color: Theme.of(context).primaryColor,
            textColor: Colors.white,
            onPressed: enableVideo,
          ),
          MaterialButton(
            minWidth: 200,
            child: Text('enable audio WebRTC'),
            color: Theme.of(context).primaryColor,
            textColor: Colors.white,
            onPressed: enableAudio,
          ),
          MaterialButton(
            minWidth: 200,
            child: Text('switch camera WebRTC'),
            color: Theme.of(context).primaryColor,
            textColor: Colors.white,
            onPressed: switchCamera,
          ),
          MaterialButton(
            minWidth: 200,
            child: Text('Mirror camera'),
            color: Theme.of(context).primaryColor,
            textColor: Colors.white,
            onPressed: mirrorCamera,
          ),
          MaterialButton(
            minWidth: 200,
            child: Text('Switch audio'),
            color: Theme.of(context).primaryColor,
            textColor: Colors.white,
            onPressed: () {
              switchAudio(QBRTCAudioOutputTypes.LOUDSPEAKER);
            },
          ),
          MaterialButton(
            minWidth: 200,
            child: Text('subscribe events WebRTC'),
            color: Theme.of(context).primaryColor,
            textColor: Colors.white,
            onPressed: () {
              subscribeEventTypes(QBRTCEventTypes.CALL);
              subscribeEventTypes(QBRTCEventTypes.CALL_END);
              subscribeEventTypes(QBRTCEventTypes.NOT_ANSWER);
              subscribeEventTypes(QBRTCEventTypes.REJECT);
              subscribeEventTypes(QBRTCEventTypes.ACCEPT);
              subscribeEventTypes(QBRTCEventTypes.HANG_UP);
              subscribeEventTypes(QBRTCEventTypes.RECEIVED_VIDEO_TRACK);
            },
          ),
          MaterialButton(
            minWidth: 200,
            child: Text('unsubscribe events WebRTC'),
            color: Theme.of(context).primaryColor,
            textColor: Colors.white,
            onPressed: () {
              unsubscribeEventTypes(QBRTCEventTypes.CALL);
              unsubscribeEventTypes(QBRTCEventTypes.CALL_END);
              unsubscribeEventTypes(QBRTCEventTypes.NOT_ANSWER);
              unsubscribeEventTypes(QBRTCEventTypes.REJECT);
              unsubscribeEventTypes(QBRTCEventTypes.ACCEPT);
              unsubscribeEventTypes(QBRTCEventTypes.HANG_UP);
              unsubscribeEventTypes(QBRTCEventTypes.RECEIVED_VIDEO_TRACK);
            },
          ),
          Container(
            margin: EdgeInsets.only(
                left: 20.0, right: 20.0, top: 20.0, bottom: 20.0),
            height: 1,
            width: double.maxFinite,
            color: Colors.grey,
          ),
          new OrientationBuilder(builder: (context, orientation) {
            return new Container(
              decoration: new BoxDecoration(color: Colors.white),
              child: new Stack(
                children: <Widget>[
                  new Align(
                    alignment: orientation == Orientation.landscape
                        ? const FractionalOffset(0.5, 0.1)
                        : const FractionalOffset(0.0, 0.5),
                    child: new Container(
                      margin: new EdgeInsets.fromLTRB(10.0, 10.0, 10.0, 10.0),
                      width: 160.0,
                      height: 160.0,
                      child: RTCVideoView(
                        onVideoViewCreated: _onLocalVideoViewCreated,
                      ),
                      decoration: new BoxDecoration(color: Colors.black54),
                    ),
                  ),
                  new Align(
                    alignment: orientation == Orientation.landscape
                        ? const FractionalOffset(0.5, 0.5)
                        : const FractionalOffset(1.0, 0.5),
                    child: new Container(
                      margin: new EdgeInsets.fromLTRB(10.0, 10.0, 10.0, 10.0),
                      width: 160.0,
                      height: 160.0,
                      child: RTCVideoView(
                        onVideoViewCreated: _onRemoteVideoViewCreated,
                      ),
                      decoration: new BoxDecoration(color: Colors.black54),
                    ),
                  ),
                ],
              ),
            );
          }),
          MaterialButton(
            minWidth: 200,
            child: Text('start rendering local'),
            color: Theme.of(context).primaryColor,
            textColor: Colors.white,
            onPressed: () {
              play();
            },
          ),
        ]))));
  }

  void _onLocalVideoViewCreated(RTCVideoViewController controller) {
    _localVideoViewController = controller;
  }

  void _onRemoteVideoViewCreated(RTCVideoViewController controller) {
    _remoteVideoViewController = controller;
  }

  Future<void> init() async {
    try {
      await QB.webrtc.init();
      SnackBarUtils.showResult(_scaffoldKey, "WebRTC was initiated");
    } on PlatformException catch (e) {
      DialogUtils.showError(context, e);
    }
  }

  Future<void> releaseWebRTC() async {
    try {
      await QB.webrtc.release();
      SnackBarUtils.showResult(_scaffoldKey, "WebRTC was released");
    } on PlatformException catch (e) {
      DialogUtils.showError(context, e);
    }
  }

  Future<void> getSessionWebRTC() async {
    try {
      QBRTCSession session = await QB.webrtc.getSession(_sessionId);
      _sessionId = session.id;
      SnackBarUtils.showResult(
          _scaffoldKey, "The session with id $_sessionId was loaded");
    } on PlatformException catch (e) {
      DialogUtils.showError(context, e);
    }
  }

  Future<void> callWebRTC() async {
    int sessionType = QBRTCSessionTypes.VIDEO;

    try {
      QBRTCSession session = await QB.webrtc.call(OPPONENTS_IDS, sessionType);
      _sessionId = session.id;
      SnackBarUtils.showResult(
          _scaffoldKey, "The call was initiated for session id: $_sessionId");
    } on PlatformException catch (e) {
      DialogUtils.showError(context, e);
    }
  }

  Future<void> acceptWebRTC() async {
    try {
      QBRTCSession session = await QB.webrtc.accept(_sessionId);
      SnackBarUtils.showResult(
          _scaffoldKey, "Session with id: $_sessionId was accepted");
    } on PlatformException catch (e) {
      DialogUtils.showError(context, e);
    }
  }

  Future<void> rejectWebRTC() async {
    try {
      QBRTCSession session = await QB.webrtc.reject(_sessionId);
      SnackBarUtils.showResult(
          _scaffoldKey, "Session with id: $_sessionId was rejected");
    } on PlatformException catch (e) {
      DialogUtils.showError(context, e);
    }
  }

  Future<void> hangUpWebRTC() async {
    try {
      QBRTCSession session = await QB.webrtc.hangUp(_sessionId);
      SnackBarUtils.showResult(
          _scaffoldKey, "Session with id: $_sessionId was hang up");
    } on PlatformException catch (e) {
      DialogUtils.showError(context, e);
    }
  }

  Future<void> enableVideo() async {
    try {
      await QB.webrtc.enableVideo(_sessionId, enable: true);
      SnackBarUtils.showResult(_scaffoldKey, "The video was enabled");
    } on PlatformException catch (e) {
      DialogUtils.showError(context, e);
    }
  }

  Future<void> enableAudio() async {
    try {
      await QB.webrtc.enableAudio(_sessionId, enable: true);
      SnackBarUtils.showResult(_scaffoldKey, "The audio was enabled");
    } on PlatformException catch (e) {
      DialogUtils.showError(context, e);
    }
  }

  Future<void> switchCamera() async {
    try {
      await QB.webrtc.switchCamera(_sessionId);
      SnackBarUtils.showResult(_scaffoldKey, "Camera was switched");
    } on PlatformException catch (e) {
      DialogUtils.showError(context, e);
    }
  }

  Future<void> mirrorCamera() async {
    try {
      await QB.webrtc.mirrorCamera(USER_ID, true);
      SnackBarUtils.showResult(_scaffoldKey, "Camera was switched");
    } on PlatformException catch (e) {
      DialogUtils.showError(context, e);
    }
  }

  Future<void> switchAudio(int output) async {
    try {
      await QB.webrtc.switchAudio(output);
      SnackBarUtils.showResult(_scaffoldKey, "Audio was switched");
    } on PlatformException catch (e) {
      DialogUtils.showError(context, e);
    }
  }

  Future<void> subscribeEventTypes(String eventName) async {
    try {
      await QB.webrtc.subscribeRTCEventTypes(eventName, (data) {});
      SnackBarUtils.showResult(_scaffoldKey, "Subscribed to $eventName");
    } on PlatformException catch (e) {
      DialogUtils.showError(context, e);
    }
  }

  Future<void> unsubscribeEventTypes(String eventName) async {
    try {
      await QB.webrtc.unsubscribeRTCEventTypes(eventName);
      SnackBarUtils.showResult(_scaffoldKey, "Unsubscribed from $eventName");
    } on PlatformException catch (e) {
      DialogUtils.showError(context, e);
    }
  }

  Future<void> play() async {
    try {
      await _localVideoViewController.play(_sessionId, USER_ID);
      await _remoteVideoViewController.play(_sessionId, OPPONENT_ID);
    } on PlatformException catch (e) {
      DialogUtils.showError(context, e);
    }
  }
}
